/*
    Indesign Calendar Wizard

    version 0.8

    by Scott Selberg
    March 11, 2006

    Script Outline:
        Variable Declarations
        User Interface
        Margins
    source references:
    Adobe InDesign CS2 Scripting Reference

   v0.5 Initial Version
   v0.6
      works in CS 1
      added mini calendars
      added multiple months (1 page per)
      added ability to fill a text frame
   v0.7
      fixed a bug in the date incrementing
      added the ability to have multiple calendars per page
   v0.8 
      added layers for tables (date, holidays, and text)
*/

/********************************************************************/
// Variable Declarations                                            
/********************************************************************/
var today       = new Date();
var miniCalendarSize = new Array( "2in", "2in", "2.75in", "2.75in" ); // 0.75 inches square
var holidays    = new Array( "1-1:New Year Day",
                             "2-14:Valentine's Day",
                             "4-1:April Fool\'s Day",
                             "7-4:Independance Day",
                             "10-31:Halloween",
                             "12-24:Christmas Eve",
                             "12-25:Christmas Day" ); 
var months      = new Array( 'January', 'February', 'March', 'April', 'May', 'June',
                             'July', 'August', 'September', 'October', 'November', 'December');
var daysLong    = new Array( 'Sunday', 'Monday', 'Tuesday', 'Wednesday',
                             'Thursday', 'Friday', 'Saturday' );
var daysMid     = new Array( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' );
var daysShort   = new Array( 'S', 'M', 'T', 'W', 'T', 'F', 'S' );

var monthsGerman      = new Array( 'Januar', 'Februar', 'Mrz', 'April', 'Mai', 'Juni',
                                   'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember');
var daysLongGerman    = new Array( 'Sonntag', 'Montag', 'Dienstag', 'Mittwoch',
                                   'Donnerstag', 'Freitag', 'Samstag' );
var daysMidGerman     = new Array( 'Son', 'Mon', 'Die', 'Mit', 'Don', 'Fre', 'Sam' );
var daysShortGerman   = new Array( 'S', 'M', 'D', 'M', 'D', 'F', 'S' );
var monthsFrench      = new Array( 'Janvier', 'Fvrier', 'Mars', 'Avril', 'Mai', 'Juin',
                                   'Juillet', 'Aot', 'Septembre', 'Octobre', 'Novembre', 'Dcembre');
var daysLongFrench    = new Array( 'Lundi', 'Mardi', 'Mercredi', 'Jeudi',
                                   'Vendredi', 'Samedi', 'Dimanche' );
var daysMidFrench     = new Array( 'Lun', 'Mar', 'Mer', 'Jeu', 'Ven', 'Sam', 'Dim' );
var daysShortFrench   = new Array( 'L', 'M', 'M', 'J', 'V', 'S', 'D' );
var monthsSpanish      = new Array( 'Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio',
                                    'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre');
var daysLongSpanish    = new Array( 'Lunes', 'Martes', 'Mircoles', 'Jueves',
                                    'Viernes', 'Sbado', 'Domingo' );
var daysMidSpanish     = new Array( 'Lun', 'Mar', 'Mi', 'Jue', 'Vie', 'Sb', 'Dom' );
var daysShortSpanish   = new Array( 'L', 'M', 'M', 'J', 'V', 'S', 'D' );

var dayOrdering = new Array( 0, 1, 2, 3, 4, 5, 6 );
var startDays   = new Array( 'Sunday', 'Monday' );
var pageTypeArray    = new Array( 'Auto', 'Current Text Frame', "0.5\" Margins", 'New Document' );
var pageOrientations = new Array( 'default', 'Portrait', 'Landscape' );
var maxRowCountArray = new Array( 'Auto', '5', '6' );
var languageArray = new Array( 'English', 'German', 'French', 'Spanish' );
var pageOrientation = pageOrientations[ 1 ];
var iMonth = today.getMonth();
var month = months[ iMonth ];
var year = today.getFullYear();
var iStartMonth = today.getMonth();
var startMonth = months[ iMonth ];
var iStartYear = today.getFullYear();
var iEndMonth = today.getMonth();
var endMonth = months[ iMonth ];
var iEndYear = today.getFullYear();
var calendarsPerPageArray = new Array( '1', '2', '3', '4', '6', '8', '9', '10', '12');
var iCalendarsPerPage;
var calendarSpacing = 0.25;
var calendarVerticalSpacing = calendarSpacing;
var calendarHorizontalSpacing = calendarSpacing;
var calendarFrameSizes;
var years = new Array( (iStartYear-3)+"",
                       (iStartYear-2)+"",
                       (iStartYear-1)+"",
                       (iStartYear  )+"",
                       (iStartYear+1)+"",
                       (iStartYear+2)+"",
                       (iStartYear+3)+"",
                       (iStartYear+4)+"",
                       (iStartYear+5)+"",
                       (iStartYear+6)+"",
                       (iStartYear+7)+"",
                       (iStartYear+8)+"",
                       (iStartYear+9)+"",
                       (iStartYear+10)+"" ); 
//var compactArray = new Array( "Include Mini Calendars", "No Mini Calendars" );
var useLayersArray = new Array( "True", "False" );
var includeHeaderArray = new Array( "Include Header in MiniCal", "Exclude Header in MiniCal" );
var bIncludeHeaderInCompact = 1;
var maxRowCount = 0; //auto
var bAddCompact = 1;
var pageType;
var frameCount = 0;
var titleSize;
var originalRulerOrigin;
var originalHorizontalMeasurementUnits;
var originalVerticalMeasurementUnits;
var calendarDatesRegExp = new RegExp( "calendarDates(.*)" )
var knownCalendars = new Array();
var bDateLayer = 0;
var bHolidaysLayer = 0;
var bTextLayer = 0;
var dateLayer;
var holidayLayer;
var textLayer;
var myLabels = new Array( "calendarDates", "calendarHolidays", "calendarText" );
var myLabel;
var iCalendarCount = 0;
var iCalendarLabelCount = 0;

var bCS      = false;
var bCSPlus  = false;
var bCS2     = false;
var bCS2Plus = false;

/********************************************************************/
// Version Diagnostic
/********************************************************************/
if (app.version == 3)
{
   bCS = true;
   bCSPlus = true;
}
else if( String(app.version).split(".")[0] == 4 )
{
   bCS2 = true;
   bCSPlus = true;
   bCS2Plus = true;
}

/********************************************************************/
// User Interface
/********************************************************************/
var myDialog = app.dialogs.add({name:"Adobe InDesign Calendar Wizard",canCancel:true});
with(myDialog)
{
   //Add a dialog column.
   with(dialogColumns.add())
   {
      with( dialogRows.add() )
      {
         with(dialogColumns.add())
         {
            staticTexts.add({staticLabel:"First Month"});
            staticTexts.add({staticLabel:"Last Month"});
         }
         with(dialogColumns.add())
         {
            var selectStartMonth = dropdowns.add({stringList:months,
               selectedIndex:today.getMonth()});
            var selectEndMonth = dropdowns.add({stringList:months,
               selectedIndex:today.getMonth()});
         }
         with(dialogColumns.add())
         {
            var selectStartYear = dropdowns.add({stringList:years,
               selectedIndex:3});
            var selectEndYear = dropdowns.add({stringList:years,
               selectedIndex:3});
         }
      }
   
      with( dialogRows.add() )
      {
         staticTexts.add({staticLabel:" "});
      }

      with(dialogRows.add())
      {
         with(dialogColumns.add())
         {
            staticTexts.add({staticLabel:"Calendar Options"});
            with( dialogRows.add() )
            {
               staticTexts.add({staticLabel:"Language"});
               var selectLanguage = dropdowns.add({stringList:languageArray,
                                                   selectedIndex:0});
            }
            with( dialogRows.add() )
            {
               staticTexts.add({staticLabel:"Start on"});
               var selectStartDay = dropdowns.add({stringList:startDays,
                                                   selectedIndex:0});
            }
            with( dialogRows.add() )
            {
               staticTexts.add({staticLabel:"Date Rows"});
               var selectMaxRowCount = dropdowns.add({stringList:maxRowCountArray,
                                                      selectedIndex:0});
            }
            with( dialogRows.add() )
            {
               var selectCompact = checkboxControls.add({checkedState:false});
               staticTexts.add({staticLabel:"Include Mini-Calendar"});
            }
         }
         with(dialogColumns.add())
         {
            staticTexts.add({staticLabel:"     "});
         }
         with(dialogColumns.add())
         {
            staticTexts.add({staticLabel:"Layer Options"});
            with(dialogRows.add())
            {
               with(dialogColumns.add())
               {
                  var selectUseCalendarLayer = checkboxControls.add({checkedState:false});
                  var selectAddHolidaysLayer = checkboxControls.add({checkedState:false});
                  var selectAddTextLayer     = checkboxControls.add({checkedState:true});
               }
               with(dialogColumns.add())
               {
                  staticTexts.add({staticLabel:"Use Calendar Layer"});
                  staticTexts.add({staticLabel:"Add Holiday Layer"});
                  staticTexts.add({staticLabel:"Add Text Layer"});
               }
            }
         }
      }
      
      with( dialogRows.add() )
      {
         staticTexts.add({staticLabel:" "});
      }

      with( dialogRows.add() )
      {
         staticTexts.add({staticLabel:"Calendars per Page"});
         var selectCalendarsPerPage = dropdowns.add({stringList:calendarsPerPageArray,
                                                     selectedIndex:0});
      }

      with( dialogRows.add() )
      {
         staticTexts.add({staticLabel:"Page"});
         var selectPageType = dropdowns.add({stringList:pageTypeArray,
                                             selectedIndex:0});
         var selectPageOrientation = dropdowns.add({stringList:pageOrientations,
                                                    selectedIndex:2});
      }

//      var selectHeader = dropdowns.add({stringList:includeHeaderArray,
//                                             selectedIndex:0});
   }
}

var myResult = myDialog.show();
if(myResult == true)
{
   iStartMonth = selectStartMonth.selectedIndex;
   startMonth = months[ iStartMonth ];
   iEndMonth = selectEndMonth.selectedIndex;
   endMonth = months[ iEndMonth ];

   startYear  = years[ selectStartYear.selectedIndex ];
   endYear    = years[ selectEndYear.selectedIndex ];
   iStartYear  = parseInt( startYear );
   iEndYear    = parseInt( endYear );

   iCalendarsPerPage = parseInt( calendarsPerPageArray[ selectCalendarsPerPage.selectedIndex ] );

   pageType        = pageTypeArray[ selectPageType.selectedIndex ];
   pageOrientation = pageOrientations[ selectPageOrientation.selectedIndex ];

   if( selectStartDay.selectedIndex == 1 )
   {
      dayOrdering = new Array(1, 2, 3, 4, 5, 6, 0);
   }

   if( selectCompact.checkedState == false )
   {
      bAddCompact = 0;
   }

   if( bAddCompact == 1 && iCalendarsPerPage > 4 )
   {
      alert( "Mini-Calendars can only be activated when there are four or less calendars per page." );
      bAddCompact = 0;
   }

   if( selectUseCalendarLayer.checkedState == true )
   {
      bDateLayer = 1;
   }
   if( selectAddHolidaysLayer.checkedState == true )
   {
      bHolidaysLayer = 1;
   }
   if( selectAddTextLayer.checkedState == true )
   {
      bTextLayer = 1;
   }

   if( selectMaxRowCount.selectedIndex > 0 )
   {
      maxRowCount = parseInt( maxRowCountArray[ selectMaxRowCount.selectedIndex ] );
   }

   if( languageArray[ selectLanguage.selectedIndex ] == "German" )
   {
      months    = monthsGerman;
      daysLong  = daysLongGerman;  
      daysMid   = daysMidGerman;   
      daysShort = daysShortGerman;
   }
   else if( languageArray[ selectLanguage.selectedIndex ] == "French" )
   {
      months    = monthsFrench;
      daysLong  = daysLongFrench;  
      daysMid   = daysMidFrench;   
      daysShort = daysShortFrench;
   }
   else if( languageArray[ selectLanguage.selectedIndex ] == "Spanish" )
   {
      months    = monthsSpanish;
      daysLong  = daysLongSpanish;  
      daysMid   = daysMidSpanish;   
      daysShort = daysShortSpanish;
   }

//   if( selectHeader.selectedIndex == 0 )
//   {
//      bIncludeHeaderInCompact = 1;
//   }
//   else
//   {
//      bIncludeHeaderInCompact = 0;
//   }

   myDialog.destroy();

   /********************************************************************/
   /* create the document                                              */
   /********************************************************************/
   var myDocument;
   var myTextFrame;
   var myPage;
   var selectionObjectType;
   var bCanUseCurrentTextFrame = false;

   try{ selectionObjectType = app.selection[0].toString(); }
   catch( myError ){ selectionObjectType = null; }

   bCanUseCurrentTextFrame = true;
/*   if( ( pageType=="Auto" || pageType == "Current Text Frame" ) &&
       iStartMonth == iEndMonth && 
       iStartYear == iEndYear && 
       iCalendarsPerPage == 1
       )
   {
      bCanUseCurrentTextFrame = true;
   }*/

   // Give the user a dialog if the settings call for the
   // text frame, but it can't be used.
   if( pageType == "Current Text Frame" )
   {
      if( !( selectionObjectType == "[object TextFrame]" || selectionObjectType == "[object InsertionPoint]" ) )
      {
         alert( "Need to select a text frame before executing the script in order to use \"Current Text Frame\"." );  
      }
      else if( !( (iStartMonth == iEndMonth ) &&
                  (iStartYear == iEndYear   ) &&
                  (iCalendarsPerPage == 1   ) ) )
      {
         alert( "When drawing into a text frame, only one month can be created." );  
         iEndMonth = iStartMonth;
         iEndYear = iStartYear;
         endMonth = startMonth;
         iCalendarsPerPage = 1;
      }
   }
   else if( pageType == "Auto" )
   {
      if( ( selectionObjectType == "[object TextFrame]" || selectionObjectType == "[object InsertionPoint]" ) &&
          !( (iStartMonth == iEndMonth ) &&
            (iStartYear == iEndYear   ) &&
            (iCalendarsPerPage == 1   )
          ) )
      {
         alert( "When drawing into a text frame, only one month can be created." );  
         iEndMonth = iStartMonth;
         iEndYear = iStartYear;
         endMonth = startMonth;
         iCalendarsPerPage = 1;
      }
   }
   else if( pageType == pageTypeArray[2] || pageType == pageTypeArray[3] )
   {
      bCanUseCurrentTextFrame = false;
   }

   if( bCanUseCurrentTextFrame && selectionObjectType == "[object TextFrame]" )
   {
      myDocument = app.activeDocument;
      addLayers( myDocument );
      myPage      = myDocument.pages.item(0);
      myTextFrame = app.selection[0];

      with(myDocument.viewPreferences)
      {
         originalRulerOrigin                = rulerOrigin;
         originalHorizontalMeasurementUnits = horizontalMeasurementUnits;
         originalVerticalMeasurementUnits   = verticalMeasurementUnits;

         rulerOrigin                = RulerOrigin.pageOrigin;
         horizontalMeasurementUnits = MeasurementUnits.inches;
         verticalMeasurementUnits   = MeasurementUnits.inches;
      }
   }
   else if( bCanUseCurrentTextFrame && selectionObjectType == "[object InsertionPoint]" )
   {
      myDocument = app.activeDocument;
      addLayers( myDocument );
      myPage      = myDocument.pages.item(0);
      if( bCS )
      {
         try{ myTextFrame = app.selection[0].parentTextFrame; }
         catch( myError )
         {
            myTextFrame = app.selection[0].parentTextFrame[0];
         }
      }
      else
      {
         myTextFrame = app.selection[0].parentTextFrames[0];
      }

      with(myDocument.viewPreferences)
      {
         originalRulerOrigin                = rulerOrigin;
         originalHorizontalMeasurementUnits = horizontalMeasurementUnits;
         originalVerticalMeasurementUnits   = verticalMeasurementUnits;

         rulerOrigin                = RulerOrigin.pageOrigin;
         horizontalMeasurementUnits = MeasurementUnits.inches;
         verticalMeasurementUnits   = MeasurementUnits.inches;
      }
   }
   else
   {
      with (app.marginPreferences)
      {
         //Save the current application default margin preferences.
         var myY1 = top;
         var myX1 = left;
         var myY2 = bottom;
         var myX2 = right;

         if( pageType == "0.5\" Margins" )
         {
            //Set the application default margin preferences.
            top    = "0.5in";
            left   = "0.5in";
            bottom = "0.5in";
            right  = "0.5in";
         }
      }
   
      myDocument  = app.documents.add();
      addLayers( myDocument );

      // Set rulers
      with(myDocument.viewPreferences)
      {
         originalRulerOrigin                = rulerOrigin;
         originalHorizontalMeasurementUnits = horizontalMeasurementUnits;
         originalVerticalMeasurementUnits   = verticalMeasurementUnits;

         rulerOrigin                = RulerOrigin.pageOrigin;
         horizontalMeasurementUnits = MeasurementUnits.inches;
         verticalMeasurementUnits   = MeasurementUnits.inches;
      }
      
      //myDocument.documentPreferences.pageWidth = "8.5i";
      //myDocument.documentPreferences.pageHeight = "11i";
      if( pageOrientation == 'Portrait')
      {
         myDocument.documentPreferences.pageOrientation = PageOrientation.portrait;
      }
      else if( pageOrientation == 'Landscape')
      {
         myDocument.documentPreferences.pageOrientation = PageOrientation.landscape;
      }
      myDocument.documentPreferences.facingPages = false;
   

      myPage      = myDocument.pages.item(0);
      if( bDateLayer == 1 )
      {
         myTextFrame = myPage.textFrames.add( dateLayer );
      }
      else
      {
         myTextFrame = myPage.textFrames.add();
      }

      calendarFrameSizes = calendarFrameBounds( myDocument, myPage, iCalendarsPerPage, calendarVerticalSpacing, calendarHorizontalSpacing );
      myTextFrame.geometricBounds = calendarFrameSizes[frameCount++];
      //myDocumentmyGetBounds( myPage, myDocument );
   
      // Reset the application default margins to their original state.
      with (app.marginPreferences)
      {
         top    = myY1;
         left   = myX1;
         bottom = myY2;
         right  = myX2;
      }
   }

   var nullSwatch  = app.activeDocument.swatches.item(0); 
   var paperSwatch = app.activeDocument.swatches.item(1); 

   /********************************************************************/
   /* create colors, layers, styles, ...                               */
   /********************************************************************/
   with(myDocument)
   {
   
      //Add colors.
      try{ colors.item("Holiday").name; }
      catch (myError){ 
         colors.add({name:"Holiday", space:ColorSpace.rgb, colorValue:[255,0,0]});
      }
      try{ colors.item("DayCellBackground").name; }
      catch (myError){ 
         colors.add({name:"DayCellBackground", space:ColorSpace.rgb, colorValue:[255,255,255]});
      }
      try{ colors.item("DayTextColor").name; }
      catch (myError){ 
         colors.add({name:"DayTextColor", space:ColorSpace.rgb, colorValue:[0,0,0]});
      }
   
      if( bAddCompact == 1 )
      {
         try{ colors.item("CompactDayCellBackground").name; }
         catch (myError){ 
            colors.add({name:"CompactDayCellBackground", space:ColorSpace.rgb, colorValue:[225,225,225]});
         }
         try{ colors.item("CompactDayTextColor").name; }
         catch (myError){ 
            colors.add({name:"CompactDayTextColor", space:ColorSpace.rgb, colorValue:[0,0,0]});
         }
      }
   
      //Add character styles
      //try{ characterStyles.item("character_style").name; }
      //catch (myError){
      //   characterStyles.add({name:"character_style"});
      //}
   
      //Add paragraph styles
      try{ paragraphStyles.item("base").name; }
      catch (myError){
         paragraphStyles.add({name:"base"});
      }
      try{ paragraphStyles.item("title").name; }
      catch (myError){
         if( iCalendarsPerPage >= 8 )
         {
            titleSize = 18;
         }
         else if( iCalendarsPerPage >= 4 )
         {
            titleSize = 24;
         }
         else
         {
            titleSize = 36;
         }
         paragraphStyles.add({name:"title", basedOn:paragraphStyles.item("base"),
                              justification:Justification.centerAlign,
                              capitalization:Capitalization.smallCaps,
                              pointSize:titleSize});
      }
      try{ paragraphStyles.item("day").name; }
      catch (myError){
         paragraphStyles.add({name:"day", basedOn:paragraphStyles.item("base"),
                              justification:Justification.centerAlign,
                              fillColor:colors.item("DayTextColor") });
      }
      try{ paragraphStyles.item("cellText").name; }
      catch (myError){
      paragraphStyles.add({name:"cellText", basedOn:paragraphStyles.item("base") });
      }
      try{ paragraphStyles.item("date").name; }
      catch (myError){
         paragraphStyles.add({name:"date", basedOn:paragraphStyles.item("base"),
                              justification:Justification.rightAlign,
                              nextStyle:paragraphStyles.item("cellText") });
      }
      try{ paragraphStyles.item("date2").name; }
      catch (myError){
         paragraphStyles.add({name:"date2", basedOn:paragraphStyles.item("date"),
                              justification:Justification.leftAlign,
                              nextStyle:paragraphStyles.item("cellText") });
      }
      try{ paragraphStyles.item("holiday").name; }
      catch (myError){
         paragraphStyles.add({name:"holiday", basedOn:paragraphStyles.item("cellText"),
                              justification:Justification.centerAlign,
                              pointSize:10,
                              fontStyle:"Italic",
                              fillColor:colors.item("Holiday") });
      }
      if( bAddCompact == 1 )
      {
         try{ paragraphStyles.item("baseCompact").name; }
         catch (myError){
            paragraphStyles.add({name:"baseCompact", basedOn:paragraphStyles.item("base"),
                                 pointSize:5});
         }
         try{ paragraphStyles.item("titleCompact").name; }
         catch (myError){
            paragraphStyles.add({name:"titleCompact", basedOn:paragraphStyles.item("baseCompact"),
                                 justification:Justification.centerAlign,
                                 capitalization:Capitalization.smallCaps });
         }
         try{ paragraphStyles.item("dayCompact").name; }
         catch (myError){
            paragraphStyles.add({name:"dayCompact", basedOn:paragraphStyles.item("baseCompact"),
                                 justification:Justification.centerAlign,
                                 fillColor:colors.item("CompactDayTextColor") });
         }
         try{ paragraphStyles.item("cellTextCompact").name; }
         catch (myError){
            paragraphStyles.add({name:"cellTextCompact", basedOn:paragraphStyles.item("baseCompact") });
         }
         try{ paragraphStyles.item("dateCompact").name; }
         catch (myError){
            paragraphStyles.add({name:"dateCompact", basedOn:paragraphStyles.item("baseCompact"),
                                 justification:Justification.centerAlign });
         }
         try{ paragraphStyles.item("date2Compact").name; }
         catch (myError){
            paragraphStyles.add({name:"date2Compact", basedOn:paragraphStyles.item("dateCompact"),
                                 justification:Justification.centerAlign });
         }
         try{ paragraphStyles.item("leftMiniCalendar").name; }
         catch (myError){
            paragraphStyles.add({name:"leftMiniCalendar", basedOn:paragraphStyles.item("baseCompact"),
                                 justification:Justification.centerAlign });
         }
         try{ paragraphStyles.item("rightMiniCalendar").name; }
         catch (myError){
            paragraphStyles.add({name:"rightMiniCalendar", basedOn:paragraphStyles.item("baseCompact"),
                                 justification:Justification.centerAlign });
         }
      }
   
      //Document baseline grid and document grid
      /*with(myDocument.gridPreferences)
      {
         baselineStart = 56;
         baselineDivision = 14;
         baselineShown = false;
         horizontalGridlineDivision = 14;
         horizontalGridSubdivision = 5
         verticalGridlineDivision = 14;
         verticalGridSubdivision = 5
         documentGridShown = false;
      }*/
   
      //Document XMP information.
      /*with (myDocument.metadataPreferences)
      {
         author = "Scott Selberg";
         copyrightInfoURL = "http://www.adobe.com";
         copyrightNotice = "This document is not copyrighted.";
      copyrightStatus = CopyrightStatus.no;
      description = "Template";
      documentTitle = "Calander Template";
      jobName = "CalanderTemplate";
      keywords = ["Calander", "Template", ""];
      var myNewContainer = createContainerItem("http://ns.adobe.com/xap/1.0/", "email");
      setProperty("http://ns.adobe.com/xap/1.0/", "email/*[1]", "okvern@adobe.com");
      }*/
   
   
      /********************************************************************/ 
      /* do the work                                                      */
      /********************************************************************/

//      myTextFrame.geometricBounds = calendarFrameBounds( myDocument, myPage, iCalendarsPerPage, calendarVerticalSpacing, calendarHorizontalSpacing )[frameCount++];
      iYear = iStartYear;
      iMonth = iStartMonth;
      var iPageCount = 1;
      var iCalendarCount = 0;

      while( ( iYear < iEndYear ) || 
             ( (iYear == iEndYear && iMonth <= iEndMonth ) ) )
      {
         if( iCalendarCount > 0 )
         {
            if( (frameCount + 1) > iCalendarsPerPage )
            {
               frameCount = 0;
               myDocument.pages.add();
               myPage      = myDocument.pages.lastItem();
            }

            if( bDateLayer == 1 )
            {
               myTextFrame = myPage.textFrames.add( dateLayer );
            }
            else
            {
               myTextFrame = myPage.textFrames.add();
            }
            myTextFrame.geometricBounds = calendarFrameSizes[ frameCount++ ];
         }

         iCalendarLabelCount = nextCalendarCount()
         myTextFrame.label = myLabels[0] + iCalendarLabelCount;
         buildCalendar( myTextFrame, iMonth, iYear.toString(), dayOrdering, maxRowCount, bAddCompact, false, bIncludeHeaderInCompact );

         var next = nextMonth( iMonth, iYear );
         iMonth = next[0];
         iYear = next[1];
         iCalendarCount++;
      }

      with(viewPreferences)
      {
         rulerOrigin                = originalRulerOrigin;
         horizontalMeasurementUnits = originalHorizontalMeasurementUnits;
         verticalMeasurementUnits   = originalVerticalMeasurementUnits;
      }
   }
}
else
{
   myDialog.destroy();
}
   
function buildCalendar( textFrame, _iMonth, _iYear, _dayOrdering, _iMaxRowCount, _bAddCompact, _bCompact, _bIncludeHeaderInCompact )
{
   var buffer;
   var days;
   var month;
   var initialRows;
   var strokeWidth;
   var frameBounds;
   var bSplitCell = 0;
   var selectedStyle;
   var myRow;
   var myCells;
   var stroke = 1/72;
   var day;

   if( _bCompact == 1 )
   {
       _iMaxRowCount = 6;
   }

   month = months[ _iMonth ];

   with( textFrame )
   {
      frameBounds = geometricBounds;
      // extra rows will be added later as needed...
      initialRows = 2;
      if( _bCompact == 1 ) 
      {
         strokeWidth = 0;
         if( _bIncludeHeaderInCompact == 0 && _iMaxRowCount > 0 )
         {
            _iMaxRowCount = _iMaxRowCount - 1;
         }
      }
      else
      {
         strokeWidth = stroke;
      }

      var myCalendar = textFrame.parentStory.tables.add({bodyRowCount: (initialRows + _iMaxRowCount), 
                                               columnCount:7,
                                               width:(frameBounds[3]-frameBounds[1]-(strokeWidth))});  // 1/72 = edge stoke width 

//      var myCalendar = parentStory.tables.item(0);
      with( myCalendar )
      {
         if( bCS )
         {
            // in CS, the table is not made properly in the constructor, so
            // I'm setting the bounds again manually...
            bodyRowCount = initialRows + _iMaxRowCount;
            columnCount = 7;
            width = frameBounds[3]-frameBounds[1]-strokeWidth;
         }

         if( ( frameBounds[3] - frameBounds[1] ) >= 7.5 )
         {
            days = daysLong;
         }
         else if( ( frameBounds[3] - frameBounds[1] ) >= 3 )
         {
            days = daysMid;
         }
         else
         {
            days = daysShort;
         }
    

        // set the title cell (month + year)
         myRow = rows.firstItem();

         // Clear formatting in compact mode...
         myCells = cells.everyItem().getElements();
         for( var i = 0; i < myCells.length; i++ )
         {
            // default the style of the paragraph.
            // this is most important in the compact mode, where
            // the default style will prevent the cell from being 
            // sized small enough
            _bCompact == 0 ? selectedStyle = "date" : selectedStyle = "dateCompact";
            myCells[i].insertionPoints.firstItem().appliedParagraphStyle = myDocument.paragraphStyles.item(selectedStyle);

            if( _bCompact == 1 )
            {
               myCells[i].topEdgeStrokeWeight    = "0in";
               myCells[i].leftEdgeStrokeWeight   = "0in";
               myCells[i].bottomEdgeStrokeWeight = "0in";
               myCells[i].rightEdgeStrokeWeight  = "0in";
               myCells[i].topInset    = (1/72) + "in";
               myCells[i].leftInset   = (1/72) + "in";
               myCells[i].bottomInset = (1/72) + "in";
               myCells[i].rightInset  = (1/72) + "in";
            }
         }

         if( _bAddCompact == 1 && _bCompact == 0 )
         {
            //myRow.cells.item(0).merge( myRow.cells.itemByRange( 0, 1 ) );
            //myRow.cells.item(1).merge( myRow.cells.itemByRange( 1, 3 ) );
            //myRow.cells.item(2).merge( myRow.cells.itemByRange( 2, 3 ) );
            myRow.cells.item(1).merge( myRow.cells.itemByRange( 1, 5 ) );
            buffer = rows.firstItem().cells.item(1);

            myRow.cells.item(0).insertionPoints.firstItem().appliedParagraphStyle = myDocument.paragraphStyles.item( "leftMiniCalendar"  );
            myRow.cells.item(2).insertionPoints.firstItem().appliedParagraphStyle = myDocument.paragraphStyles.item( "rightMiniCalendar" );

            myRow.cells.everyItem().leftEdgeStrokeWeight = "0in";
            myRow.cells.everyItem().topEdgeStrokeWeight = "0in";
            myRow.cells.everyItem().rightEdgeStrokeWeight = "0in";
            
            var miniLeftTextFrame = myPage.textFrames.add();
            miniLeftTextFrame.geometricBounds = ( miniCalendarSize );
            var next = previousMonth( _iMonth, _iYear );
            buildCalendar( miniLeftTextFrame, next[0], next[1], _dayOrdering, 6, false, true, _bIncludeHeaderInCompact );
            app.select( miniLeftTextFrame );
            app.cut();
            app.select( myRow.cells.item(0).insertionPoints.firstItem() );
            app.paste();

            var miniRightTextFrame = myPage.textFrames.add();
            miniRightTextFrame.geometricBounds = ( miniCalendarSize );
            var next = nextMonth( _iMonth, _iYear );
            buildCalendar( miniRightTextFrame, next[0], next[1], _dayOrdering, 6, false, true, _bIncludeHeaderInCompact );
            app.select( miniRightTextFrame );
            app.cut();
            app.select( myRow.cells.item(2).insertionPoints.firstItem() );
            app.paste();

            app.select( NothingEnum.nothing );
         }
         else
         {
            myRow.cells.item(0).merge( myRow.cells.itemByRange( 0, 6 ) );
            buffer = rows.firstItem().cells.item(0);
         }
         buffer.contents = month + " " + _iYear;
         _bCompact == 0 ? selectedStyle = "title" : selectedStyle = "titleCompact";
         buffer.paragraphs.item(0).appliedParagraphStyle = myDocument.paragraphStyles.item( selectedStyle);

         // this call makes the cell render, so that the height may be asked for
         buffer = rows.firstItem().cells.firstItem().contents;  

         // set the day header cells
         if( _bCompact == 0 || ( _bCompact == 1 && _bIncludeHeaderInCompact == 1 ) )
         {
            myRow   = rows.nextItem( myRow );
            myCells = myRow.cells;
            for( var i = 0; i < myCells.length; i++ )
            {
               myCells.item(i).contents = days[(_dayOrdering[i])];
               _bCompact == 0 ? selectedStyle = "DayCellBackground" : selectedStyle = "CompactDayCellBackground";
               myCells.item(i).fillColor = myDocument.colors.item(selectedStyle);
               _bCompact == 0 ? selectedStyle = "day" : selectedStyle = "dayCompact";
               myCells.item(i).paragraphs.item(0).appliedParagraphStyle = myDocument.paragraphStyles.item(selectedStyle);
               myCells.item(i).verticalJustification = VerticalJustification.centerAlign;
            }
            // this call makes the cell render, so that the height may be asked for
            buffer = myCells.firstItem().contents;
         }

         // figure out how many days in the month, and what day of the week to start on
         var date = new Date()
         date.setFullYear( parseInt(_iYear), _iMonth, 1 );
         var startDay = date.getDay();
         var endDay = GetDaysInMonth( _iMonth, parseInt(_iYear) );
         var daysInTheMonth = new Array();
         for( var i = 0; i < endDay; i++ )
         {
            daysInTheMonth[i] = (endDay - i) + "";
         }

         // start the days...
         var bStarted = 0;
         myRow   = rows.nextItem( myRow );
         try{ myCells = myRow.cells; }
         catch( myError ) {
               myRow = rows.add();
               myRow.cells.everyItem().label = "";
               if( bCS )
               {
                  myRow = rows.lastItem();
               }
               myCells = myRow.cells;
         }
         for( var i = 0; i < myCells.length; i++ )
         {
            myCells.item(i).fillColor = nullSwatch;

            if( bStarted == 1 || startDay == _dayOrdering[i] )
            {
               bStarted = 1;
               day = daysInTheMonth.pop()
               myCells.item(i).contents = day;
               myCells.item(i).label = (iMonth+1).toString() + "-" + day;
            }
            else
            {
                myCells.item(i).contents = ' ';
            }

            _bCompact == 0 ? selectedStyle = "date" : selectedStyle = "dateCompact";
            myCells.item(i).paragraphs.item(0).appliedParagraphStyle = myDocument.paragraphStyles.item(selectedStyle);

            if( _bCompact == 1 )
            {
               myCells.item(i).verticalJustification = VerticalJustification.centerAlign;
            }
            else
            {
               myCells.item(i).verticalJustification = VerticalJustification.topAlign;
            }

         }            

         // continue the days...
         while( daysInTheMonth.length > 0 )
         {
            myRow = rows.nextItem( myRow );
            try{ myCells = myRow.cells; }
            catch( myError ) {
               if( _iMaxRowCount == 0 )
               {
                  myRow = rows.add();
                  myRow.cells.everyItem().label = "";
                  if( bCS )
                  {
                     myRow = rows.lastItem();
                  }
                  myCells = myRow.cells;
               }
               else
               {
                  bSplitCell = 1;
                  myRow = rows.lastItem();
                  myCells = myRow.cells;
               }
            }
            for( var i = 0; i < myCells.length; i++ )
            {
               if( daysInTheMonth.length > 0 )
               {
                  if( bSplitCell == 0 )
                  {
                     day = daysInTheMonth.pop()
                     myCells.item(i).contents = day;
                     myCells.item(i).label = (iMonth+1).toString() + "-" + day;

                     _bCompact == 0 ? selectedStyle = "date" : selectedStyle = "dateCompact";
                     myCells.item(i).paragraphs.item(0).appliedParagraphStyle = myDocument.paragraphStyles.item(selectedStyle);

                     if( _bCompact == 1 )
                     {
                        myCells.item(i).verticalJustification = VerticalJustification.centerAlign;
                     }
                     else
                     {
                        myCells.item(i).verticalJustification = VerticalJustification.topAlign;
                     }
                  }
                  else
                  {
                     myCells.item(i).topLeftDiagonalLine = true;
                     myCells.item(i).verticalJustification = VerticalJustification.justifyAlign;
                     myCells.item(i).contents = myCells.item(1).contents + "\r" + daysInTheMonth.pop();
                     _bCompact == 0 ? selectedStyle = "date" : selectedStyle = "dateCompact";
                     myCells.item(i).paragraphs.item(0).appliedParagraphStyle = myDocument.paragraphStyles.item(selectedStyle);
                     _bCompact == 0 ? selectedStyle = "date2" : selectedStyle = "date2Compact";
                     myCells.item(i).paragraphs.item(1).appliedParagraphStyle = myDocument.paragraphStyles.item(selectedStyle);
                  }
               }
            }            
         }

         // adjust the cell height

         myCells = columns.firstItem().cells;
         var strokeHeight = (myCells[ myCells.length-1 ].bottomEdgeStrokeWeight)/72;
         var cellHeight;
         var rowStart;

         if( bCS )
         {
            // in CS, the bottomEdgeStrokeWeight doesn't seem to work, so I'm hard
            // coding it to 1/72
            strokeHeight = stroke;
         }

         if( _bCompact == 1 && _bIncludeHeaderInCompact == 0 ) 
         {
            rowStart = 1;
            cellHeight = ((frameBounds[2] - frameBounds[0] - myCells[0].height - strokeHeight)/(myCells.length - 1));
         }
         else
         {
            rowStart = 2;
            cellHeight = ((frameBounds[2] - frameBounds[0] - myCells[0].height - myCells[1].height  - strokeHeight)/(myCells.length - 2));
         }
      
         for( i = rowStart; i < myCells.length; i++ )
         {
            myCells[i].height = cellHeight;
         }
      }
   }

   if( bHolidaysLayer == 1 && _bCompact != 1)
   {
      var holidaysTextFrame = myPage.textFrames.add( holidaysLayer );
      holidaysTextFrame.label = myLabels[1] + iCalendarLabelCount;
      holidaysTextFrame.geometricBounds = textFrame.geometricBounds;

      holidaysCalendar = holidaysTextFrame.parentStory.tables.add({bodyRowCount:myCalendar.bodyRowCount, 
                                                                   columnCount:myCalendar.columnCount
                                                                 });

      if( bCS )
      {
         // in CS, the table is not made properly in the constructor, so
         // I'm setting the bounds again manually...
         holidaysCalendar.bodyRowCount = myCalendar.bodyRowCount;
         holidaysCalendar.columnCount = myCalendar.columnCount;
      }

      var myMasterRow = myCalendar.rows.lastItem().cells;
      var mySlaveRow  = holidaysCalendar.rows.lastItem().cells;

      for( i = 0; i < myMasterRow.length; i++ )
      {
         mySlaveRow[i].width = myMasterRow[i].width;
      }

      var myMasterColumn = myCalendar.columns.firstItem().cells;
      var mySlaveColumn  = holidaysCalendar.columns.firstItem().cells;

      for( i = 0; i < myMasterColumn.length; i++ )
      {
         mySlaveColumn[i].height = myMasterColumn[i].height;
      }

      mySlaveRow  = holidaysCalendar.rows.firstItem();
      mySlaveRow.cells.item(1).merge( mySlaveRow );

      holidaysCalendar.cells.everyItem().leftEdgeStrokeWeight = "0in";
      holidaysCalendar.cells.everyItem().topEdgeStrokeWeight = "0in";
      holidaysCalendar.cells.everyItem().rightEdgeStrokeWeight = "0in";
      holidaysCalendar.cells.everyItem().bottomEdgeStrokeWeight = "0in";
      holidaysCalendar.cells.everyItem().insertionPoints.firstItem().appliedParagraphStyle = myDocument.paragraphStyles.item("holiday");
      holidaysCalendar.cells.everyItem().verticalJustification = VerticalJustification.bottomAlign;

      // disabled because it causes the myCalendar to change layers
      //addHolidayText( myCalendar, holidaysCalendar );
   }

   if( bTextLayer == 1 && _bCompact != 1)
   {
      var textTextFrame = myPage.textFrames.add( textLayer );
      textTextFrame.label = myLabels[2] + iCalendarLabelCount;
      textTextFrame.geometricBounds = textFrame.geometricBounds;

      textCalendar = textTextFrame.parentStory.tables.add({bodyRowCount:myCalendar.bodyRowCount, 
                                                           columnCount:myCalendar.columnCount
                                                         });

      if( bCS )
      {
         // in CS, the table is not made properly in the constructor, so
         // I'm setting the bounds again manually...
         textCalendar.bodyRowCount = myCalendar.bodyRowCount;
         textCalendar.columnCount = myCalendar.columnCount;
      }

      var myMasterRow = myCalendar.rows.lastItem().cells;
      var mySlaveRow  = textCalendar.rows.lastItem().cells;

      for( i = 0; i < myMasterRow.length; i++ )
      {
         mySlaveRow[i].width = myMasterRow[i].width;
      }

      var myMasterColumn = myCalendar.columns.firstItem().cells;
      var mySlaveColumn  = textCalendar.columns.firstItem().cells;

      for( i = 0; i < myMasterColumn.length; i++ )
      {
         mySlaveColumn[i].height = myMasterColumn[i].height;
      }

      mySlaveRow  = textCalendar.rows.firstItem();
      mySlaveRow.cells.item(1).merge( mySlaveRow );

      textCalendar.cells.everyItem().leftEdgeStrokeWeight = "0in";
      textCalendar.cells.everyItem().topEdgeStrokeWeight = "0in";
      textCalendar.cells.everyItem().rightEdgeStrokeWeight = "0in";
      textCalendar.cells.everyItem().bottomEdgeStrokeWeight = "0in";
      textCalendar.cells.everyItem().insertionPoints.firstItem().appliedParagraphStyle = myDocument.paragraphStyles.item("cellText");
   }
}

//myGetBounds calculates and return the bounds of the "live area" of the page.
function myGetBounds(myPage, myDocument)
{
	var array = new Array()
	var item = 0;
	with (myDocument.documentPreferences)
   {
		var myPageHeight = pageHeight;
		var myPageWidth = pageWidth;
	}

	with(myPage.marginPreferences)
    {
		var myX1 = left;
		var myY1 = top;
		var myY2 = bottom;
		var myX2 = right;
	}

	array[item++] = myY1;
	array[item++] = myX1;
	array[item++] = myPageHeight - myY2;
	array[item++] = myPageWidth - myX2;

   return array;
}

function GetDaysInMonth(iMonth, iYear) 
{
   //Returns the number of days in a month
   /*
      Solve for the start of first day of the next month
      subtract half day 
      find the date of the last day.
   */
   
   if( iMonth == 11 )
   {
      iYear++;
      iMonth = 0;
   }
   else
   {
      iMonth++;
   }

   var dateBuffer = new Date(iYear, iMonth, 1);
   var dateBuffer2 = new Date( dateBuffer.getTime() - (60*60*(12)*1000) );
   return dateBuffer2.getDate();
}
function nextMonth( iMonth, iYear )
{
   var iNextMonth;
   var iNextYear;
   if( iMonth == 11 )
   {
      iNextMonth = 0;
      iNextYear = iYear + 1;
   }
   else
   {
      iNextMonth = iMonth + 1;
      iNextYear = iYear;
   }

   return( new Array( iNextMonth, iNextYear ) );
}
function previousMonth( iMonth, iYear )
{
   var iNextMonth;
   var iNextYear;
   if( iMonth == 0 )
   {
      iNextMonth = 12;
      iNextYear = iYear - 1;
   }
   else
   {
      iNextMonth = iMonth - 1;
      iNextYear = iYear;
   }

   return( new Array( iNextMonth, iNextYear ) );
}

function calendarFrameBounds(document, page, framesPerPage, frameVerticalSpacing, frameHorizontalSpacing  )
{
   var pageOrientation = document.documentPreferences.pageOrientation;

   var frameBounds = new Array()
   var frames      = new Array()
	var i = 0;
   var j = 0;
   var frameCount      = 0
   var framesPerRow    = 1;
   var framesPerColumn = 1;

	with (document.documentPreferences)
   {
		var myPageHeight = pageHeight;
		var myPageWidth = pageWidth;
	}

	with(page.marginPreferences)
   {
		var myX1 = left;
		var myY1 = top;
		var myY2 = myPageHeight - bottom;
		var myX2 = myPageWidth - right;
   }

   if( framesPerPage == 1 )
   {
      framesPerRow    = 1;
      framesPerColumn = 1;
   }
   else if( ( framesPerPage == 2 || framesPerPage == 3 ) && myPageHeight >= myPageWidth )
   {
      framesPerRow    = 1;
      framesPerColumn = framesPerPage;
   }
   else if( ( framesPerPage == 2 || framesPerPage == 3 ) && myPageHeight < myPageWidth )
   {
      framesPerRow    = framesPerPage;
      framesPerColumn = 1;
   }
   else if( framesPerPage == 4 )
   {
      framesPerRow    = 2;
      framesPerColumn = 2;
   }
   else if( ( framesPerPage == 6 || framesPerPage == 8 || framesPerPage == 10 ) && myPageHeight >= myPageWidth )
   {
      framesPerRow    = 2;
      framesPerColumn = framesPerPage/2;
   }
   else if( ( framesPerPage == 6 || framesPerPage == 8 || framesPerPage == 10 ) && myPageHeight < myPageWidth )
   {
      framesPerRow    = framesPerPage/2;
      framesPerColumn = 2;
   }
   else if( framesPerPage == 9 )
   {
      framesPerRow    = 3;
      framesPerColumn = 3;
   }
   else if( framesPerPage == 12 && myPageHeight >= myPageWidth )
   {
      framesPerRow    = 3;
      framesPerColumn = 4;
   }
   else if( framesPerPage == 12 && myPageHeight < myPageWidth )
   {
      framesPerRow    = 4;
      framesPerColumn = 3;
   }

   var dimensionY = (myY2 - myY1 - (frameVerticalSpacing * (framesPerColumn - 1)))/framesPerColumn;
   var dimensionX = (myX2 - myX1 - (frameHorizontalSpacing * (framesPerRow - 1)))/framesPerRow;

   for( i=0; i<framesPerColumn; i++ )
   {
      frameBounds[0] = myY1 + (i * frameVerticalSpacing ) + (i+0) * dimensionY;
      frameBounds[2] = myY1 + (i * frameVerticalSpacing ) + (i+1) * dimensionY;

      for( j=0; j<framesPerRow; j++ )
      {
         frameBounds[1] = myX1 + (j * frameHorizontalSpacing ) + (j+0) * dimensionX;
         frameBounds[3] = myX1 + (j * frameHorizontalSpacing ) + (j+1) * dimensionX;

         frames[frameCount++] = new Array(frameBounds[0], frameBounds[1], frameBounds[2], frameBounds[3]);
      }
   }

//   alert( frames.toString() );
   return frames;
}


function addLayers( myDocument )
{
   with( myDocument )
   {
      // Add layers
      if( bDateLayer == 1 )
      {
         try{ dateLayer = layers.item("calendar").name; }
         catch (myError){
            dateLayer = layers.add({name:"calendar"});
         }
      }
      if( bHolidaysLayer == 1 )
      {
         try{ holidaysLayer = layers.item("calHolidays").name; }
         catch (myError){
            holidaysLayer = layers.add({name:"calHolidays"});
         }
      }
      if( bTextLayer == 1 )
      {
         try{ textLayer = layers.item("calText").name; }
         catch (myError){
            textLayer = layers.add({name:"calText"});
         }
      }   
   }

   return;
}

function findCalendars()
{
   var knownCalendars = new Array();
   var buffer;
   var myError;

   var myDocument = app.activeDocument;
   var myTextFrames = myDocument.textFrames;

   for( var i = 0; i < myTextFrames.length; i++ )
   {
      buffer = calendarDatesRegExp.exec( myTextFrames.item(i).label );
      try{ knownCalendars.push( parseInt( buffer[1] ) ) }
      catch( myError ){ }
   }

   return( knownCalendars )
}

function nextCalendarCount( )
{
   var buffer = 0;
   try
   {
      var knownCalendars = findCalendars();
      for( var i = 0; i < knownCalendars.length; i++ )
      {
         if( knownCalendars[i] > buffer )
         {
            buffer = knownCalendars[i];
         }
      }      
   } catch( myError )
   {}

   return( buffer + 1 )
}

function addHolidayText( calendarTable, holidaysTable )
{
   var myCalendarRow;
   var myHolidaysRow;
   var myCalendarCells;
   var myHolidaysCells;
   var calendarDate;

   // Title row (January 2006) - nothing to do
   myCalendarRow = calendarTable.rows.firstItem();
   myHolidaysRow  = holidaysTable.rows.firstItem();

   // Header row (Sunday, Monday, ...) - nothing to do
   myCalendarRow = calendarTable.rows.nextItem( myCalendarRow );
   myHolidaysRow = holidaysTable.rows.nextItem( myHolidaysRow );

   // First day row
   myCalendarRow = calendarTable.rows.nextItem( myCalendarRow );
   myHolidaysRow = holidaysTable.rows.nextItem( myHolidaysRow );

   try
   {
      while( myCalendarRow.cells.length >= 0 )
      {
         myCalendarCells = myCalendarRow.cells;
         myHolidaysCells = myHolidaysRow.cells;

         for( var i = 0; i < myCalendarCells.length; i++ )
         {
            try
            {
               calendarDate = myCalendarCells[i].label
               for( var j = 0; j < holidays.length; j++ )
               {
                  var holidaysBuffer = new Array();
                  holidaysBuffer = holidays[j].split(":");
                  if( calendarDate == holidaysBuffer[0] )
                  {
                     // This command causes the calendarTable to move to 
                     // the layer with the holidaysTable.  This is not
                     // desireable, and the feature is not much, so forget it.
                     //myHolidaysCells[i].contents = holidaysBuffer[1];
                  }
               }
            }
            catch( myError ){}
         }

         myCalendarRow = calendarTable.rows.nextItem( myCalendarRow );
         myHolidaysRow = holidaysTable.rows.nextItem( myHolidaysRow );
      }
   }
   catch( myError ){}

}


